

from netgen.csg import *
from ngsolve import *

geo = CSGeometry()
br1 = OrthoBrick (Pnt(0,0,0), Pnt(1,1,1)).bc("outerbnd")
br2 = OrthoBrick (Pnt(0.2,0.2,0.2), Pnt(0.8, 0.8, 0.8)).bc("innerbnd")
geo.Add (br2.mat("inner"))
geo.Add ( (br1-br2).mat("outer") )

mesh = Mesh(geo.GenerateMesh(maxh=0.3))
fesCurl = HCurl(mesh, nograds = True, complex = True, order = 2)

normals = CoefficientFunction ([None, specialcf.normal(3)] )
bfv_normals = BoundaryFromVolumeCF(normals)

cfrotation = CoefficientFunction((-y-1j*y, x+1j*x, 0))
gfrotation = GridFunction(fesCurl)
gfrotation.Set(cfrotation)
curlrotation = curl(gfrotation)
bfcurl = BoundaryFromVolumeCF(curlrotation)
print("curl Integral = ", Integrate(Norm(bfcurl*bfv_normals), mesh, definedon=mesh.Boundaries("innerbnd")))



cfz = CoefficientFunction((0, 0, 1 + 1j))
bfz = BoundaryFromVolumeCF(cfz)
print("cfz  Integral = ", Integrate(Norm(bfz*bfv_normals), mesh, definedon=mesh.Boundaries("innerbnd")))

